package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptWithOrderRes;
import com.diagbot.entity.Concept;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconRSTypeEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.ConceptServiceImpl;
import com.diagbot.util.IntegerUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.ParamConvertUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptUsualVO;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/5/6 15:04
 */
@Component
public class ConceptFacade extends ConceptServiceImpl {

    /**
     * 验证存在并且获取概念基本信息
     *
     * @param conceptBaseVO 概念名称和类型
     * @return 概念基本信息
     */
    public Concept exist(ConceptBaseVO conceptBaseVO) {
        QueryWrapper<Concept> conceptQueryWrapper = new QueryWrapper<>();
        conceptQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("lib_name", conceptBaseVO.getName())
                .eq("lib_type", conceptBaseVO.getLibType());
        Concept concept = this.getOne(conceptQueryWrapper);
        if (concept == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "概念不存在");
        }
        return concept;
    }

    /**
     * 根据名称和词性获取概念Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语和术语id Map
     */
    public Map<String, Long> getConceptMap(ConceptSearchVO conceptSearchVO) {
        //入参验证
        if (ListUtil.isEmpty(conceptSearchVO.getNameList())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "术语列表不能为空");
        }

        if (IntegerUtil.isNull(conceptSearchVO.getType())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "类型不能为空");
        }

        List<Integer> inputTyps = new ArrayList<>();
        inputTyps.add(ConceptTypeEnum.Symptom.getKey());
        inputTyps.add(ConceptTypeEnum.Vital.getKey());
        inputTyps.add(ConceptTypeEnum.Lis.getKey());
        inputTyps.add(ConceptTypeEnum.Pacs.getKey());
        inputTyps.add(ConceptTypeEnum.Disease.getKey());

        if (!inputTyps.contains(conceptSearchVO.getType())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "类型超出范围");
        }
        //获取结构
        List<Concept> concepts
                = this.getListByNamesAndType(conceptSearchVO.getNameList(),
                ParamConvertUtil.conceptConvert2Lib(conceptSearchVO.getType()));

        //出参封装
        Map<String, Long> map = new HashMap<>();
        if (ListUtil.isNotEmpty(concepts)) {
            map = concepts.stream().collect(Collectors.toMap(r -> r.getLibName(), r -> r.getId()));
        }
        return map;
    }

    /**
     * 根据名称和词性获取概念列表
     *
     * @param nameList
     * @param libType
     * @return
     */
    public List<Concept> getListByNamesAndType(List<String> nameList, Integer libType) {
        QueryWrapper<Concept> conceptQueryWrapper = new QueryWrapper<>();
        conceptQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("lib_name", nameList)
                .eq("lib_type", libType);
        List<Concept> list = this.list(conceptQueryWrapper);
        return list;
    }

    /**
     * 获取常用标签
     *
     * @param conceptUsualVO
     * @return
     */
    public List<ConceptBaseDTO> getConceptUsual(ConceptUsualVO conceptUsualVO) {
        //入参验证
        List<Integer> inputTyps = new ArrayList<>();
        inputTyps.add(ConceptTypeEnum.Symptom.getKey());
        inputTyps.add(ConceptTypeEnum.Vital.getKey());
        inputTyps.add(ConceptTypeEnum.Lis.getKey());
        inputTyps.add(ConceptTypeEnum.Pacs.getKey());

        if (!inputTyps.contains(conceptUsualVO.getType())) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "超出范围");
        }

        //获取标签内容
        ConceptWrapper conceptWrapper = new ConceptWrapper();
        conceptWrapper.setStartName(conceptUsualVO.getDeptName());//科室名称
        conceptWrapper.setStartType(LexiconTypeEnum.DEPARTMENT.getKey());
        conceptWrapper.setRelationType(LexiconRSTypeEnum.ORDER_BY.getKey());
        conceptWrapper.setEndAge(conceptUsualVO.getAge());//年龄
        conceptWrapper.setEndSex(conceptUsualVO.getSexType());//性别
        //找出标签类型对应的类型id
        conceptWrapper.setEndType(ParamConvertUtil.conceptConvert2Lib(conceptUsualVO.getType()));//标签类型
        List<ConceptWithOrderRes> concepts = this.getConceptWithOrder(conceptWrapper);
        List<ConceptBaseDTO> list = new ArrayList();
        //封装
        ConceptBaseDTO conceptBaseDTO = new ConceptBaseDTO();
        for (ConceptWithOrderRes conceptWithOrderRes : concepts) {
            conceptBaseDTO = new ConceptBaseDTO();
            conceptBaseDTO.setConceptId(conceptWithOrderRes.getEndId());
            conceptBaseDTO.setName(conceptWithOrderRes.getEndName());
            list.add(conceptBaseDTO);
        }
        return list;
    }
}
